--[[
                            Average.lua & Average.lc


    App zur zur Mittelwertsbestimmung von Telemetriedaten mit Hilfe der
    rekursive Berechnung des arithmetischen Mittels.

    Quelle: https://de.wikipedia.org/wiki/Arithmetisches_Mittel#Unabh%C3%A4ngig_und_identisch_verteilte_Zufallsvariablen

    Die Benutzung der App zur Mittelwertbestimmung geschieht auf eigene Gefahr.
    Ich übernehme keine Verantwortung oder Haftung bei jeglichem Schaden oder V
    erletzung, die in Zusammenhang mit der Benutzung dieser Mittelwertbestimmungs
    App entstehen könnte. Sobald Ihr die App auf eure Fernbedienung kopiert habt
    und benutzt, seid Ihr hiermit einverstanden.

    ----------------------------------------------------------------------------
    Lizenz: Darf gerne geteilt werden und geändert werden, solange es nicht
            commerziell benutzt wird
    ----------------------------------------------------------------------------


    App to calculate the Arithmetic mean of telemetry values by means of the
    rekursive method to calculate the Arithmetic mean.

    Reference: https://de.wikipedia.org/wiki/Arithmetisches_Mittel#Unabh%C3%A4ngig_und_identisch_verteilte_Zufallsvariablen

    The App is without ANY warranty of any kind and it´s usage is at your own risk.
    I will not be held accountable or responsible for any type of damage to person
    or property that may happend while using the App.  If you copy and install
    the App on your transmitter is only because you have agreed to the above.

    ----------------------------------------------------------------------------
    License: Share as you like. Can be used and changed non commercial only
    ----------------------------------------------------------------------------


    Aplicación para calcular el promedio (media aritmética) de la serie de valores
    reportados via telemetria mediante el uso del cálculo recursivo de la media
    aritmética.

    Referencia: https://de.wikipedia.org/wiki/Arithmetisches_Mittel#Unabh%C3%A4ngig_und_identisch_verteilte_Zufallsvariablen

    El uso de esta aplicación es vajo riesgo y responsivilidad propia. No se
    asumira ningun tipo de responsivilidad por cualquier tipo de daño a perso, propiedad
    y terceros, que pueda surjir durante el usa de esta aplicación. Si usted copia
    e instala esta aplicacion a su control, es porque usted asume la responsivilidad
    anteriormente explicada.

    ----------------------------------------------------------------------------
    Licencia: Comparta la aplicación como quiera. Esta aplicación puede ser usada
    y modificaca a comveniencia, siempre y cuando su uso no sea comercial.
    ----------------------------------------------------------------------------


                                                                  2021. Rodrigo
--]]


------Variables
local Avg_Ver

local Avg_lang
local Avg_n0, Avg_n1, Avg_x1, Avg_avg1, Avg_avg0, Avg_val
local Avg_min, Avg_max, Avg_t0, Avg_t1, Avg_te, Avg_j
local Avg_sw, Avg_psSw, Avg_out
local Avg_rnd, Avg_plyd, Avg_blnk = true, false, false
local Avg_t = {h=0, m=0, s=0}

local Avg_sensVal
local Avg_sens = {  selec = 0,
                    id = 0,
                    param = 0,
                    label = " ",
                    unit = " ",
                    dec = " "
                  }

local Avg_lst = { t   = {nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil},
                  psd = {nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil},
                  min = {nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil},
                  max = {nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil},
                  val = {nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil},
                }

local Avg_tMilli = system.getTimeCounter
local getTime = system.getTime
local getSensorValueByID = system.getSensorValueByID
local getInputsVal = system.getInputsVal
local getSwitchInfo = system.getSwitchInfo
local format = string.format
local insert = table.insert
local remove = table.remove
local getTextWidth = lcd.getTextWidth
local drawText = lcd.drawText
local drawLine = lcd.drawLine
local playFile = system.playFile
local playNumber = system.playNumber


--------------------------------------------------------------------------------
-- Read TRANSLATION
local function getTranslation()
  local trans = system.getLocale()
  local file  = io.readall("Apps/Average/lang/Average.jsn")
  local dec_file = json.decode(file)

  if (dec_file) then
    Avg_lang = dec_file[trans] or dec_file[dec_file.default]
  end

collectgarbage()
end
-- Read TRANSLATION
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
--Draw App TELEMETRY WINDOW-----------------------------------------------------

--Display the calculated Average Value
local function AvgTele1(w, h)

  if (h==24) then --SMALL Telemetry Window
    drawText(145-getTextWidth(FONT_BIG, Avg_val.." "..Avg_sens.unit),1, Avg_val.." "..Avg_sens.unit, FONT_BIG)

  else --BIG Telemetry Window
    drawLine(3, 35, 145, 35) --Hori. line
    drawLine(75, 38, 75, 65) --Vert. Line

    drawText(147-(getTextWidth(FONT_BOLD, Avg_sens.unit)+getTextWidth(FONT_MAXI, Avg_val)), -3, Avg_val, FONT_MAXI)
    drawText(147-getTextWidth(FONT_BOLD, Avg_sens.unit),14, Avg_sens.unit, FONT_BOLD)

    drawText(3,36, Avg_lang.rntime, FONT_MINI)

    --Show blinking PAUSED and renove the runtime if messaurement has been paused
    if (Avg_blnk==true) then
      drawText(38-getTextWidth(FONT_BOLD, Avg_lang.paused)/2, 47, Avg_lang.paused, FONT_BOLD)
    else
      drawText(38-getTextWidth(FONT_NORMAL, format("%02d:%02d",Avg_t.m,Avg_t.s))/2,47, format("%02d:%02d",Avg_t.m,Avg_t.s), FONT_NORMAL)
    end

    drawText(78,36, Avg_lang.mnmx, FONT_MINI)
    drawText(147-getTextWidth(FONT_MINI, Avg_max..Avg_sens.unit),46, Avg_max..Avg_sens.unit, FONT_MINI)
    drawText(147-getTextWidth(FONT_MINI, Avg_min..Avg_sens.unit),56, Avg_min..Avg_sens.unit, FONT_MINI)
  end

end --AvgTele1(w, h)


--Display the last 10 calculated Averages
local function AvgTele2()
  local y

  drawLine(75, 2, 75, 65)   --Vertical Line

    --Left side of the TelemetryWindow
    y = 0
    for i = 1, 5 do

      if (Avg_j>10) then
        i = i + (Avg_j-10)
      end

      if (Avg_lst.t[i]) then
        drawText(28-getTextWidth(FONT_MINI, Avg_lst.t[i]), 1+(y*13.5), Avg_lst.t[i], FONT_MINI)

        if (Avg_lst.psd[i]==true) then
          drawText(27, 1+(y*13.5), "p", FONT_MINI)
        end

        drawText(73-getTextWidth(FONT_MINI, Avg_lst.val[i]), 1+(y*13.5), Avg_lst.val[i], FONT_MINI)

        y = y + 1
      end
    end

    --Right side of the TelemetryWindow
    y = 0
    for i = 6, 10 do

      if (Avg_j>10) then
        i = i + (Avg_j-10)
      end

      if (Avg_lst.t[i]) then
        drawText(105-getTextWidth(FONT_MINI, Avg_lst.t[i]), 1+(y*13.5), Avg_lst.t[i], FONT_MINI)

        if (Avg_lst.psd[i]==true) then
          drawText(104, 1+(y*13.5), "p", FONT_MINI)
        end

        drawText(148-getTextWidth(FONT_MINI, Avg_lst.val[i]), 1+(y*13.5), Avg_lst.val[i], FONT_MINI)

        y = y + 1
      end
    end

end --AvgTele2

--Draw App TELEMETRY WINDOW-----------------------------------------------------
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- App Functions

-- Convert from Milliseconds to hh:mm:ss
local function toTime(mSec)
local h, m, s

  s  = mSec / 1000
  h  = s // 3600
  m  = s//60 - (h*60)

  s  = (s % 60)//1

collectgarbage()
return h, m, s     -- Return in hh:mm:ss format
end --toTime(mSec)

-- App Functions
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
--Draw App FORM-----------------------------------------------------------------
local function AvgForm(subform)
  local pSave = system.pSave
  local registerTelemetry = system.registerTelemetry
  local setTitle = form.setTitle
  local setFocusedRow = form.setFocusedRow
  local setButton = form.setButton
  local addRow = form.addRow
  local addSpacer = form.addSpacer
  local addLabel = form.addLabel
  local addInputbox = form.addInputbox
  local addSelectbox = form.addSelectbox


  --SUBFORM 1  --Show all calculated values
  if (subform==1) then
    setButton(1, ":browser", HIGHLIGHTED)
    setButton(2, ":tools", ENABLED)

    setTitle(Avg_lang.hdr)

    --Test if average values are present and if yes, display them
    if (Avg_lst.val[1]==nil) then
      addSpacer(1,50)

      addRow(2)
      addSpacer((290 - getTextWidth(FONT_BOLD, Avg_lang.noData))/2 ,1)
      addLabel({label= Avg_lang.noData, font=FONT_BOLD, enabled=false, width=320})

      addSpacer(1,45)
    else
      local Avg_lstTxt, Avg_txtL

      Avg_lstTxt = "---  "..Avg_sens.label.." ["..Avg_sens.unit.."]  ---"
      Avg_txtL = getTextWidth(FONT_BOLD, Avg_lstTxt)

      addRow(2)
      addSpacer(150-(Avg_txtL/2),0)
      addLabel({label= Avg_lstTxt, font=FONT_BOLD, enabled=false})

      addSpacer(1,7)

      for i in ipairs(Avg_lst.val) do
        addRow(6)

        Avg_lstTxt = i..".   "..Avg_lst.t[i]
        addLabel({label= Avg_lstTxt, font=FONT_NORMAL, width=85,  alignRight=true, enabled=false})

        addSpacer(-9,0)
        if (Avg_lst.psd[i]==true) then
          addLabel({label= "p", font=FONT_MINI, width=15, alignRight=true, enabled=false})
        else
          addLabel({label= " ", font=FONT_MINI, width=15, alignRight=true, enabled=false})
        end

        if (Avg_lst.min[i]>=0) then
          Avg_lstTxt = " "..format(Avg_sens.dec, Avg_lst.min[i])
        else
          Avg_lstTxt = format(Avg_sens.dec, Avg_lst.min[i])
        end

        if (Avg_lst.max[i]>=0) then
          Avg_lstTxt = Avg_lstTxt.." / "..format(Avg_sens.dec, Avg_lst.max[i])
        else
          Avg_lstTxt = Avg_lstTxt.." /"..format(Avg_sens.dec, Avg_lst.max[i])
        end

        Avg_txtL = getTextWidth(FONT_NORMAL, Avg_lstTxt)
        addSpacer(60-(Avg_txtL/2),0)
        addLabel({label= Avg_lstTxt, font=FONT_NORMAL, width=125, enabled=false})

        addLabel({label= Avg_lst.val[i], font=FONT_BOLD, width=150, alignRight=true, enabled=false})

      end

    end --Avg_lst.val[1]==nil

    addSpacer(1,10)

    addRow(1)
    addLabel({label="by Rodrigo - v." .. Avg_Ver, font=FONT_MINI, alignRight=true, enabled=false})

  collectgarbage()
  end --SUBFORM 1


  --SUBFORM 2 -- App Settings
  if (subform==2) then
    setButton(1, ":browser", ENABLED)
    setButton(2, ":tools", HIGHLIGHTED)

    -- Read ALL AVIABLE SENSORS (only if 2.Form menu is active to preserve memory at runtime)
    local getSensors = system.getSensors()
    local sensorName = ""
    local sensorList = {  name = {},
                          id = {},
                          param = {}
                        }

    for i, sensor in ipairs(getSensors) do
      if (sensor.label~="") then

        if (sensor.param==0) then
          sensorName = sensor.label..": "
        else
          insert(sensorList.name, sensorName..sensor.label)
          insert(sensorList.id, sensor.id)
          insert(sensorList.param, sensor.param)
        end

      end
    end


    -- Build the 2. Form
    setTitle(Avg_lang.appName)

    addSpacer(1,5)
    addRow(1)
    addLabel({label=Avg_lang.mainsttgs, font=FONT_BOLD})
    addSpacer(1,2)

    addRow(2)
    addLabel({label=Avg_lang.selSnsr, font=FONT_NORMAL})
    addSelectbox(sensorList.name, Avg_sens.selec, true, function(value)
      Avg_sens.selec = value
      Avg_sens.id = sensorList.id[value]
      Avg_sens.param = sensorList.param[value]

      pSave ("Avg_sens.selec", Avg_sens.selec)
      pSave ("Avg_sens.id", Avg_sens.id)
      pSave ("Avg_sens.param", Avg_sens.param)

      --Get the label, unit & number of decimals from the selected Sensor
      local Avg_sensData = system.getSensorByID(Avg_sens.id, Avg_sens.param)

      if (Avg_sens.id~=0 and Avg_sensData~=nil) then
        Avg_sens.label = Avg_sensData.label
        Avg_sens.unit = Avg_sensData.unit
        Avg_sens.dec = "%."..Avg_sensData.decimals.."f"
      else
        Avg_sens.label = " "
        Avg_sens.unit = " "
        Avg_sens.dec = "%.0f"
      end

      --Redraw the TelemetryWindow to update Label & Units
      registerTelemetry(1, Avg_lang.appName..": "..Avg_sens.label, 0, AvgTele1)

    end)

    addRow(2)
    addLabel({label=Avg_lang.strstp, font=FONT_NORMAL})
    addInputbox(Avg_sw, false, function(value)
      Avg_sw = value
      pSave ("Avg_sw", Avg_sw)
    end)

    addSpacer(1,7)
    addLabel({label=Avg_lang.extras, font=FONT_BOLD})
    addSpacer(1,2)

    addRow(2)
    addLabel({label=Avg_lang.psSwitch, font=FONT_NORMAL})
    addInputbox(Avg_psSw, false, function(value)
      Avg_psSw = value
      pSave ("Avg_psSw", Avg_psSw)
    end)

    addRow(2)
    addLabel({label=Avg_lang.voice, font=FONT_NORMAL})
    addInputbox(Avg_out, false, function(value)
      Avg_out = value
      pSave ("Avg_out", Avg_out)
    end)
    addSpacer(1,10)

    addRow(1)
    addLabel({label="by Rodrigo - v." .. Avg_Ver, font=FONT_MINI, alignRight=true})

    setFocusedRow(4)

  collectgarbage()
  end --SUBFORM 2


end
--Draw App FORM-----------------------------------------------------------------
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
--Re-Init FORM when Functions Buttons F1-F5 are PRESSED
local function keyPressed(key)
  local reinit = form.reinit
  local setFocusedRow = form.setFocusedRow


  if (key == KEY_1) then  --Show all calculated values
    reinit(1)
    setFocusedRow(1)
  end
  if (key == KEY_2) then  -- App Settings
    reinit(2)
    setFocusedRow(1)
  end

end
--Re-Init FORM when Functions Buttons F1-F5 are PRESSED
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
--The LOOP----------------------------------------------------------------------
local function loop()

  --Exit loop if no Sensor (check via the id) or no Start/stop switch
  --has been choosen
  if (Avg_sens.id==0 or Avg_sw==nil or getSwitchInfo(Avg_sw).assigned==false) then
    return
  end


  --Say current Average Value by voice
  if (getInputsVal(Avg_out)==1 and Avg_plyd==false) then
    playFile (Avg_lang.sndFile, AUDIO_QUEUE)
    playNumber(Avg_val, 2, Avg_sens.unit)

    Avg_plyd = true
  elseif (getInputsVal(Avg_out)==-1 and Avg_plyd==true) then
    Avg_plyd = false
  end


  --Exit loop if messaurement has been paused
  if (Avg_psSw~=nil and getSwitchInfo(Avg_psSw).assigned==true and getInputsVal(Avg_psSw)==1) then
    Avg_lst.psd[Avg_j+1] = true --For displaying a "P" in the last calculated TelemetryWindow

    --timer offset <-- during the pause
    Avg_t0 = Avg_tMilli()
    Avg_t1 =  Avg_te

    --Runtime to create the blinking PAUSED text
    if ((getTime()%2)==0) then
      Avg_blnk = true
    else
      Avg_blnk = false
    end

    return --EXIT THE LOOP
  else
    Avg_blnk = false
  end


  --Read selected Sensor
  Avg_sensVal = getSensorValueByID(Avg_sens.id, Avg_sens.param)
  if (Avg_sensVal and Avg_sensVal.valid) then
    Avg_x1 = Avg_sensVal.value
  else
    Avg_x1 = 0
  end


  if (getInputsVal(Avg_sw)==1) then
    if (Avg_rnd==true) then --if calculation has runned (Avg_rnd)

      --Reset all Values
      Avg_t0 = Avg_tMilli()
      Avg_t1 = 0
      Avg_t = {h=0, m=0, s=0}
      Avg_n0 = 0
      Avg_n1 = 1
      Avg_avg0 = 0
      Avg_avg1 = 0
      Avg_min = Avg_x1
      Avg_max = Avg_x1
      Avg_rnd = false
    end

    --Calculate the recursive Average Avg_avg1
    Avg_avg1 = (Avg_avg0*(Avg_n0/Avg_n1))+(Avg_x1/Avg_n1)

    --Round calculated value (Avg_avg1) to the same number of decimals (Avg_sens.dec)
    --as the Sensor reports
    Avg_val = format(Avg_sens.dec, Avg_avg1)

    Avg_n0 = Avg_n1
    Avg_n1 = Avg_n1 + 1

    Avg_avg0 = Avg_avg1

    if (Avg_x1<Avg_min) then Avg_min=Avg_x1 end
    if (Avg_x1>Avg_max) then Avg_max=Avg_x1 end

    --Runtime & conversion to min:sec
    Avg_te = (Avg_tMilli() - Avg_t0) + Avg_t1

    Avg_t.h, Avg_t.m, Avg_t.s = toTime(Avg_te)
    Avg_t.m = (Avg_t.h*60) + Avg_t.m --to display hours as minutes --> hours won´t be displayed on TelemetryWindow

  --Calculation ended --> Avarage Value voice output
  elseif (Avg_rnd==false and getInputsVal(Avg_sw)~=1) then
    playFile (Avg_lang.sndFile, AUDIO_QUEUE)
    playNumber(Avg_val, 2, Avg_sens.unit)

    --Save all data to display it on the "Last 10 Values TelemetryWindow"
    insert(Avg_lst.t, format("%02d:%02d",Avg_t.m,Avg_t.s))
    insert(Avg_lst.min, Avg_min)
    insert(Avg_lst.max, Avg_max)
    insert(Avg_lst.val, Avg_val)

    if (Avg_lst.psd[Avg_j]~=true) then
      Avg_lst.psd[Avg_j] = false
    end

    Avg_j = Avg_j + 1
    Avg_rnd = true
  end

collectgarbage()
end
--The LOOP----------------------------------------------------------------------
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
--App INITIALIZATION------------------------------------------------------------
local function init()
  local pLoad = system.pLoad
  local registerTelemetry = system.registerTelemetry
  local registerForm = system.registerForm

  Avg_sens.selec = pLoad("Avg_sens.selec", 0)
  Avg_sens.id = pLoad("Avg_sens.id", 0)
  Avg_sens.param = pLoad("Avg_sens.param", 0)

  --Get the label, unit & number of decimals from the selected Sensor
  local Avg_sensData = system.getSensorByID(Avg_sens.id, Avg_sens.param)

  if (Avg_sens.id~=0 and Avg_sensData~=nil) then
    Avg_sens.label = Avg_sensData.label
    Avg_sens.unit = Avg_sensData.unit
    Avg_sens.dec = "%."..Avg_sensData.decimals.."f"
  else
    Avg_sens.label = " "
    Avg_sens.unit = " "
    Avg_sens.dec = "%.0f"
  end


  Avg_sw = pLoad("Avg_sw")
  Avg_psSw = pLoad("Avg_psSw")
  Avg_out = pLoad("Avg_out")


  Avg_t = {h=0, m=0, s=0}
  Avg_t0 = 0
  Avg_t1 = 0
  Avg_n0 = 0
  Avg_n1 = 1
  Avg_avg0 = 0
  Avg_avg1 = 0
  Avg_x1 = 0
  Avg_val = 0
  Avg_min = 0
  Avg_max = 0
  Avg_j = 0


  registerTelemetry(1, Avg_lang.appName..": "..Avg_sens.label, 0, AvgTele1)
  registerTelemetry(2, Avg_lang.appName..": "..Avg_lang.lstVal, 2, AvgTele2)

  registerForm(1, MENU_APPS, Avg_lang.appName, AvgForm, keyPressed)

end
--------------------------------------------------------------------------------
Avg_Ver = "1.0"

getTranslation()
collectgarbage()
return {init=init, loop=loop, author="Rodrigo", version=Avg_Ver, name=Avg_lang.appName}
